@@ -28,6 +28,7 @@ class Transformer(object):
         sequence_length=None,  # 是否固定序列长度
         keep_tokens=None,  # 要保留的词ID列表
         compound_tokens=None,  # 扩展Embedding
+        residual_attention_scores=False,  # Attention矩阵加残差
         layers=None,  # 外部传入的Keras层
         prefix=None,  # 层名前缀
         name=None,  # 模型名称
@@ -52,6 +53,8 @@ class Transformer(object):
         self.compound_tokens = compound_tokens
         self.attention_bias = None
         self.position_bias = None
+        self.attention_scores = None
+        self.residual_attention_scores = residual_attention_scores
         self.layers = {} if layers is None else layers
         self.prefix = prefix or ''
         self.name = name
@@ -119,6 +122,9 @@ class Transformer(object):
         if layer is Dropout and self.dropout_rate == 0:
             return inputs
 
+        if layer is MultiHeadAttention and self.residual_attention_scores:
+            kwargs['return_attention_scores'] = True
+
         arguments = arguments or {}
         name = self.prefixed(kwargs.get('name'))
         kwargs['name'] = name
@@ -132,11 +138,27 @@ class Transformer(object):
         else:
             if isinstance(self.layers[name], MultiHeadAttention):
                 if name in self.attention_caches:
+                    # 如果检测到Cache的传入，那么自动在Key,Value处拼接起来
                     k_cache, v_cache = self.attention_caches[name]
                     k_name, v_name = name + '-Cached-Key', name + '-Cached-Value'
                     k = Concatenate1D(name=k_name)([k_cache, inputs[1]])
                     v = Concatenate1D(name=v_name)([v_cache, inputs[2]])
                     inputs = inputs[:1] + [k, v] + inputs[3:]
+                if self.residual_attention_scores:
+                    # 如果使用残差Attention矩阵，则给每个Attention矩阵加上前上一层的Attention
+                    # 矩阵，这对应RealFormer设计（https://arxiv.org/abs/2012.11747）。目前
+                    # 该实现还相对粗糙，可能欠缺通用性。
+                    if self.attention_scores is not None:
+                        if arguments.get('a_bias'):
+                            a_bias = Add(name=name + '-Attention-Bias'
+                                        )([inputs[3], self.attention_scores])
+                        else:
+                            a_bias = self.attention_scores
+                        inputs = inputs[:3] + [a_bias] + inputs[4:]
+                        arguments['a_bias'] = True
+                    o, a = self.layers[name](inputs, **arguments)
+                    self.attention_scores = a
+                    return o
             return self.layers[name](inputs, **arguments)
 
     def get_inputs(self):
@@ -212,13 +234,19 @@ class Transformer(object):
     def load_embeddings(self, embeddings):
         """处理Embedding层权重
         """
+        embeddings = embeddings.astype(K.floatx())  # 防止np.average报错
+
         if self.keep_tokens is not None:
             embeddings = embeddings[self.keep_tokens]
 
         if self.compound_tokens is not None:
-            ext_embeddings = np.array([
-                embeddings[idxs].mean(0) for idxs in self.compound_tokens
-            ])
+            ext_embeddings = []
+            for item in self.compound_tokens:
+                if isinstance(item, list):
+                    item = (item, [1] * len(item))
+                ext_embeddings.append(
+                    np.average(embeddings[item[0]], 0, item[1])
+                )
             embeddings = np.concatenate([embeddings, ext_embeddings], 0)
 
         return embeddings
@@ -231,10 +259,13 @@ class Transformer(object):
         else:
             return tf.train.load_variable(checkpoint, name)
 
-    def create_variable(self, name, value):
+    def create_variable(self, name, value, dtype=None):
         """创建一个变量
         """
-        return K.variable(self.initializer(value.shape), name=name), value
+        dtype = dtype or K.floatx()
+        return K.variable(
+            self.initializer(value.shape, dtype), dtype, name=name
+        ), value
 
     def variable_mapping(self):
         """构建keras层与checkpoint的变量名之间的映射表
@@ -281,7 +312,7 @@ class Transformer(object):
 
         K.batch_set_value(weight_value_pairs)
 
-    def save_weights_as_checkpoint(self, filename, mapping=None):
+    def save_weights_as_checkpoint(self, filename, mapping=None, dtype=None):
         """根据mapping将权重保存为checkpoint格式
         """
         mapping = mapping or self.variable_mapping()
@@ -294,7 +325,7 @@ class Transformer(object):
                 layer = self.layers[layer]
                 values = K.batch_get_value(layer.trainable_weights)
                 for name, value in zip(variables, values):
-                    variable, value = self.create_variable(name, value)
+                    variable, value = self.create_variable(name, value, dtype)
                     all_variables.append(variable)
                     all_values.append(value)
             with tf.Session() as sess:
@@ -657,12 +688,12 @@ class BERT(Transformer):
         else:
             return variable
 
-    def create_variable(self, name, value):
+    def create_variable(self, name, value, dtype=None):
         """在tensorflow中创建一个变量
         """
         if name == 'cls/seq_relationship/output_weights':
             value = value.T
-        return super(BERT, self).create_variable(name, value)
+        return super(BERT, self).create_variable(name, value, dtype)
 
     def variable_mapping(self):
         """映射到官方BERT权重格式
@@ -1619,12 +1650,12 @@ class T5_Base(Transformer):
         else:
             return variable
 
-    def create_variable(self, name, value):
+    def create_variable(self, name, value, dtype=None):
         """在tensorflow中创建一个变量
         """
         if 'relative_attention_bias' in name:
             value = value.T
-        return super(T5_Base, self).create_variable(name, value)
+        return super(T5_Base, self).create_variable(name, value, dtype)
 
     def variable_mapping(self):
         """映射到官方T5权重格式
