@@ -5,6 +5,7 @@ import unicodedata, re
 from bert4keras.snippets import is_string, is_py2
 from bert4keras.snippets import open
 from bert4keras.snippets import convert_to_unicode
+from bert4keras.snippets import truncate_sequences
 
 
 def load_vocab(dict_path, encoding='utf-8', simplified=False, startswith=None):
@@ -95,7 +96,7 @@ class TokenizerBase(object):
 
         if maxlen is not None:
             index = int(self._token_end is not None) + 1
-            self.truncate_sequence(maxlen, tokens, None, -index)
+            truncate_sequences(maxlen, -index, tokens)
 
         return tokens
 
@@ -109,23 +110,6 @@ class TokenizerBase(object):
         """
         return [self.token_to_id(token) for token in tokens]
 
-    def truncate_sequence(
-        self, maxlen, first_sequence, second_sequence=None, pop_index=-1
-    ):
-        """截断总长度
-        """
-        if second_sequence is None:
-            second_sequence = []
-
-        while True:
-            total_length = len(first_sequence) + len(second_sequence)
-            if total_length <= maxlen:
-                break
-            elif len(first_sequence) > len(second_sequence):
-                first_sequence.pop(pop_index)
-            else:
-                second_sequence.pop(pop_index)
-
     def encode(
         self, first_text, second_text=None, maxlen=None, pattern='S*E*E'
     ):
@@ -148,7 +132,8 @@ class TokenizerBase(object):
             second_tokens = second_text
 
         if maxlen is not None:
-            self.truncate_sequence(maxlen, first_tokens, second_tokens, -2)
+            index = int(self._token_end is not None) + 1
+            truncate_sequences(maxlen, -index, first_tokens, second_tokens)
 
         first_token_ids = self.tokens_to_ids(first_tokens)
         first_segment_ids = [0] * len(first_token_ids)
@@ -186,7 +171,9 @@ class Tokenizer(TokenizerBase):
     """Bert原生分词器
     纯Python实现，代码修改自keras_bert的tokenizer实现
     """
-    def __init__(self, token_dict, do_lower_case=False, **kwargs):
+    def __init__(
+        self, token_dict, do_lower_case=False, word_maxlen=200, **kwargs
+    ):
         super(Tokenizer, self).__init__(**kwargs)
         if is_string(token_dict):
             token_dict = load_vocab(token_dict)
@@ -195,6 +182,7 @@ class Tokenizer(TokenizerBase):
         self._token_dict = token_dict
         self._token_dict_inv = {v: k for k, v in token_dict.items()}
         self._vocab_size = len(token_dict)
+        self._word_maxlen = word_maxlen
 
         for token in ['pad', 'unk', 'mask', 'start', 'end']:
             try:
@@ -285,24 +273,24 @@ class Tokenizer(TokenizerBase):
     def _word_piece_tokenize(self, word):
         """word内分成subword
         """
-        if word in self._token_dict:
+        if len(word) > self._word_maxlen:
             return [word]
 
-        tokens = []
-        start, stop = 0, 0
+        tokens, start, end = [], 0, 0
         while start < len(word):
-            stop = len(word)
-            while stop > start:
-                sub = word[start:stop]
+            end = len(word)
+            while end > start:
+                sub = word[start:end]
                 if start > 0:
                     sub = '##' + sub
                 if sub in self._token_dict:
                     break
-                stop -= 1
-            if start == stop:
-                stop += 1
-            tokens.append(sub)
-            start = stop
+                end -= 1
+            if start == end:
+                return [word]
+            else:
+                tokens.append(sub)
+                start = end
 
         return tokens
 
